This directory contains the program for the experiments of the CIKM 2023 full paper 
"On the Thresholding Strategy for Infrequent Labels in Multi-label Classification."

# Prerequisites
+ The algorithms are written in MATLAB. Therefore, MATLAB installation is required.
+ The program for aggregating the experimental results is written in Python. 
  There, Python with packages `pandas` and `scipy` is required.
+ Because the LIBLINEAR components have to be compiled, compilers `gcc` and `g++` are required.
+ Use the command `./setup_liblinear.sh` to download LIBLINEAR and compile the required dependencies.
+ Datasets: 
    + Use `./get_datasets.sh` to download the 6 datasets used in the paper. 
    + The file `data_paths.m` records the position of the training and testing files. 
      There is no need to change it if you use `get_datasets.sh`.

# Reproducing Results from the Paper
+ Reproduce the main results (Table 2), the label order experiments (Supplementary D, Table 8,9) or the bias term experiment (Table 7):
    + Open MATLAB and run the script `run_main_exp.m`. The random seeds, datasets, and algorithms to be run can be configured in the file beforehand. 
    + To run the main experiments, run the file `run_main_exp.m` without modification. 
    + To run the label order experiment, select the corresponding methods in the `methods` variable.
    + To run the algorithms without the bias term for the bias term experiment, remove `-B 1` from the `options` variable.
    + When specifying datasets, the names used should be the same as those keys in `data_paths.m`.
    + The names of all algorithms should be specified the same as those in the variable `methods` defined in `run_main_exp.m`. Notice that binary relevance will always be run regardless of whether it is specified or not.
    + After running the experiments, the trained model and the testing result will be stored in the folder `./exp_outputs/{dataset}_{seed}/{LIBLINEAR options}/`.
    + To calculate the statistics from all runs and perform statistical tests. Use `python aggregate_results.py`. See the file for details.
+ Plotting Figure 1 and Figure 2:
    + Open MATLAB and run the script `fig12.m`. By configuring the variable `l` in the script, Figure 1 or Figure 2 can be plotted. See the file for details.
+ Plotting Figure 3:
    + After running the main experiments. Run `p_vs_f_all.m` with MATLAB. The script calculates the curve of F-measure (averaged over different random seeds) versus p (#positive samples) for each dataset and algorithm. The results are placed in `./exp_outputs/p_vs_F/{dataset}_{algorithm}_({LIBLINEAR options})_p-vs-F.csv`.
    + Then, use `python subtract.py` to calculate the difference between these curves. The results are CSV files named `{dataset}_{algo2}-{algo1}_({options}).csv` that can be used to plot Figure 3. 
+ Reproduce experiments from Supplementary E (Table 10,11):
    + Run the script `count_fig12.m` with MATLAB. By configuring the `dataname` variable the results for different datasets can be produced.