% Plot the value of F-measure corresponding to different threshold
addpath('utils', 'algos')

%% load data
run('data_paths.m')
paths = datasets('wiki31k');

[y, x, map] = libsvmread_ml(paths{1});
[~, tx, ~] = libsvmread_ml(paths{2});

% prepare input
bias = 1;
x = x';
tx = tx';

[x, tx] = fill_x(x, tx, bias);
y = y * 2 - 1;

%% configuration
options = '-q'; % the bias term is already added

% choose label
% plot fig 1
l = 10001;
% plot fig 2
% l = 1;

% plot smoothed F-measure or use FBR to select threshold
b = 0;
a = 0;
fbr = 0;

%%
d = size(y,2);
w = zeros(size(x,1), d);

n = size(x,2);
split = round(n * 2 / 3);
valid_x = x(:, split:end);
valid_y = y(split:end, l);
scut_w = do_train(y(1:split-1,l), x(:, 1:split-1), options);

wTx = valid_x' *  scut_w; % shape = (#sample in valid)

if a ~= 0 || b~= 0
    f_type = 'Smoothed F-measure';
else
    f_type = 'F-measure';
end

% start thresholding

scut_b = 0;
start_F = fmeasure(valid_y, 2*(wTx > -scut_b)-1);

% for each label (column) sort by predicted score
[sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 

% best_F is initialized as predicting all as 1,
% that is, the threshhold is set to the lowest
tp = sum(valid_y == 1); fp = size(valid_y,1)-tp; fn = 0;
cut = 0;
best_F = (a+2*tp) / (b+2*tp + fp + fn);

Fs = zeros(size(valid_y,1)+1, 1);
Fs(1) = best_F;

for i = 1:size(valid_y, 1)
    if valid_y(sorted_wTx_index(i)) == -1
        fp = fp - 1;
    else
        tp = tp - 1; fn = fn + 1;
    end
    F = (a+2*tp) / (b+2*tp + fp + fn);
    if isnan(F)
        F = 0;
    end
    Fs(i+1) = F;
    
    if F >= best_F
        best_F = F;
        cut = i+1;
    end
end            

if best_F < fbr
    cut = size(valid_y,1) + 1;
end

% plot distribution

figure('Name', [num2str(l) '-' num2str(a) ',' num2str(b)])

yyaxis left
edges = linspace(min(wTx), max(wTx), 100);
h1 = histogram(wTx(valid_y < 0), edges, 'FaceAlpha', 0.5, 'FaceColor', '#0072BD', 'DisplayName', 'negative distribution');
max1 = max(h1.Values);
hold all
h2 = histogram(wTx(valid_y > 0), edges, 'FaceAlpha', 0.5, 'FaceColor', '#D95319', 'DisplayName', 'positive distribution');
set(gca, 'YScale', 'log');
yl = ylim;
ylim([0.1 yl(2)])
ylabel('#samples in each bin')


% plot threshold
cut_vals = [sorted_wTx(1)-1e-3; (sorted_wTx(1:end-1)+sorted_wTx(2:end))/2; sorted_wTx(end)+1e-3];
th_plot = plot([cut_vals(cut), cut_vals(cut)], ylim, 'LineStyle', '-', 'Color', 'magenta', 'Marker', 'None');


% plot (smoothed) F-measure
yyaxis right
ylabel(f_type)
% reduce the amount of plot points for F-measure
grouped_cut_vals = round(cut_vals * 1000) / 1000;
Fs = grpstats(Fs, grouped_cut_vals, 'max');
grouped_cut_vals = unique(grouped_cut_vals);
F_plot = plot(grouped_cut_vals, Fs, 'lineWidth', 2, 'LineStyle','-', 'Color', '#D95319');

% legends and axis setting
if max(Fs) > 0
    ylim([-0.005*max(F_plot.YData) 1.2 * max(F_plot.YData)])
else
    ylim([-0.005 1])
end

xlabel('validation decision values')
ax = gca; % ax is the handle to the axes
% ax.Color = 'none';
ax.YAxis(1).Color = 'k';
ax.YAxis(2).Color = 'k';

if best_F < fbr
    purple_legend = 'Threshold from FBR';
else
    purple_legend = ['Threshold maximizing ' f_type];
end

handles = [h1, h2, F_plot, th_plot];
desc = ["negative distribution", ...
    "positive distribution", ...
    f_type, ...
    purple_legend];

legend(handles, desc)

if l == 1
    ylim([0 0.27])
end


hold off
saveas(gcf, ['./' num2str(l) '_f_' num2str(a) ',' num2str(b) '.jpg'])

