import os
import pandas as pd

# where the csvs generated from `p_vs_f_all.m`` are located
csv_dir = './exp_outputs/p_vs_F'

# results on these datasets will be calculated
datasets = [
    'ppi',
    'flickr',
    'blogcat',
    'rcv1v2',
    'eurlex',
    'wiki31k'
]

# results with this LIBLINEAR option will be calculated
options = '-q -B 1'

# calculate improvement relative to this algorithm
base_algo = 'SCutFBR'

# compare these algorithms to base_algo
compare_algos = ['smooth-all', 'SCutFBRn']

### END OF CONFIGURATION ###

def output_diff(dataset, options, base_algo, compare_algos):
    base_file = os.path.join(csv_dir, f'{dataset}_{base_algo}_({options})_p-vs-F.csv')
    
    if not os.path.isfile(base_file):
        print(f"ERROR: File {base_file} not found")
        return
    
    base_df = pd.read_csv(base_file, index_col=0)

    for algo in compare_algos:
        compare_file = os.path.join(csv_dir, f'{dataset}_{algo}_({options})_p-vs-F.csv')
        if not os.path.isfile(base_file):
            print(f"ERROR: File {base_file} not found")
            continue

        compare_df = pd.read_csv(compare_file, index_col=0)

        diff = (compare_df - base_df) * 100
        diff_file = os.path.join(csv_dir, f'{dataset}_{algo}-{base_algo}_({options}).csv')
        diff.to_csv(diff_file)
        print(f"{diff_file} outputted.")

for data in datasets:
    output_diff(data, options, base_algo, compare_algos)

